@@ -1,7 +1,7 @@
 package Perinci::CmdLine;
 
-our $DATE = '2014-10-14'; # DATE
-our $VERSION = '1.34'; # VERSION
+our $DATE = '2014-10-24'; # DATE
+our $VERSION = '1.35'; # VERSION
 
 use 5.010001;
 #use strict; # enabled by Moo
@@ -47,7 +47,10 @@ has riap_client => (
         require Perinci::Access;
         require Perinci::Access::Perl;
         require Perinci::Access::Schemeless;
-        my %args = %{$self->riap_client_args // {}};
+        my %args = (
+            riap_version => $self->riap_version,
+            %{$self->riap_client_args // {}},
+        );
         my %opts;
         if ($self->undo) {
             $opts{use_tx} = 1;
@@ -621,20 +624,25 @@ sub hook_display_result {
     my $handle = $r->{output_handle};
 
     # determine whether to binmode(STDOUT,":utf8")
-    my $utf8 = $ENV{UTF8};
-    if (!defined($utf8)) {
+    my $utf8;
+    {
+        last if defined($utf8 = $ENV{UTF8});
+        if ($resmeta->{'x.hint.result_binary'}) {
+            # XXX only when format is text?
+            $utf8 = 0; last;
+        }
         my $am = $self->action_metadata->{$r->{action}}
             if $r->{action};
-        $utf8 //= $am->{use_utf8};
-    }
-    if (!defined($utf8) && $r->{subcommand_data}) {
-        $utf8 //= $r->{subcommand_data}{use_utf8};
-    }
-    $utf8 //= $self->use_utf8
-        if $self->can("use_utf8");
-    if ($utf8) {
-        binmode(STDOUT, ":utf8");
+        last if defined($utf8 //= $am->{use_utf8});
+
+        if ($r->{subcommand_data}) {
+            last if defined($utf8 //= $r->{subcommand_data}{use_utf8});
+        }
+
+        $utf8 //= $self->use_utf8
+            if $self->can("use_utf8");
     }
+    binmode(STDOUT, ":utf8") if $utf8;
 
     if ($ENV{COMP_LINE} || $res->[3]{"cmdline.skip_format"}) {
         print $handle $res->[2];
@@ -845,7 +853,7 @@ Perinci::CmdLine - Rinci/Riap-based command-line application framework
 
 =head1 VERSION
 
-This document describes version 1.34 of Perinci::CmdLine (from Perl distribution Perinci-CmdLine), released on 2014-10-14.
+This document describes version 1.35 of Perinci::CmdLine (from Perl distribution Perinci-CmdLine), released on 2014-10-24.
 
 =head1 SYNOPSIS
 
